﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using InterfaceComm;
using MongoDB.Driver;
using MongoDB.Driver.Builders;



/* ==============================================================================
 * 作者：xjh 
 * 博客地址：http://www.cnblogs.com/qtqq/
 * 希望有更好的方法一起交流下，谢谢~~~
 * ==============================================================================*/

namespace Data.NoSql
{

    public class MongodbBase<T> where T : class,IMongoEntity
    {
        protected MongoServer server = null;
        protected MongoDatabase db = null;
        protected MongoCollection<T> collection;
        protected void Init(string DbName)
        {
            var Item = ManagerConfig.ServiceSettings.mongodbs.Where(p => p.dbName == DbName).FirstOrDefault();
            if (Item == null)
            {
                throw new Exception("不存在数据库为: " + DbName + " 的配置对象，请检查");
            }
            else
            {
                server = MongoDB.Driver.MongoServer.Create(Item.hostName);
                db = server.GetDatabase(Item.dbName);
                collection = db.GetCollection<T>(typeof(T).Name.Replace("Entity", ""));
            }
        }

        #region 查询
        /// <summary>
        /// 根据ID获取对象
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public T GetModelById(string id)
        {
            return collection.FindOneById(id);
        }

        /// <summary>
        /// 获取一条记录(自定义条件)
        /// </summary>
        /// <returns></returns>
        public T FirstOrDefault(Expression<Func<T, bool>> expression)
        {
            MongoDB.Driver.IMongoQuery query = Query<T>.Where(expression);
            return collection.FindOne(query);
        }

        /// <summary>
        /// 获取一条记录
        /// </summary>
        /// <returns></returns>
        public T FirstOrDefault()
        {
            return collection.FindAll().FirstOrDefault();
        }

        /// <summary>
        /// 获取全部
        /// </summary>
        /// <returns></returns>
        public List<T> FindAll()
        {
            return collection.FindAll().ToList();
        }

        /// <summary>
        /// 获取全部(自定义条件)
        /// </summary>
        /// <returns></returns>
        public List<T> FindAll(Expression<Func<T, bool>> expression)
        {
            MongoDB.Driver.IMongoQuery query = Query<T>.Where(expression);
            return collection.Find(query).ToList();
        }

        /// <summary>
        /// 根据条件获取数量
        /// </summary>
        /// <param name="expression"></param>
        /// <returns></returns>
        public long GetCount(Expression<Func<T, bool>> expression = null)
        {
            if (expression == null)
            {
                return collection.Count();
            }
            else
            {
                return collection.Count(Query<T>.Where(expression));
            }
        }

        /// <summary>
        /// 根据ID判断是否存在
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool Exists(string id)
        {
            return collection.FindOneById(id) != null;
        }

        /// <summary>
        /// 分页
        /// </summary>
        /// <param name="PageIndex">总页码</param>
        /// <param name="PageSize">页容量</param>
        /// <param name="RowCounts">总记录数</param>
        /// <param name="expression">条件</param>
        /// <param name="IsAsc">是否是正序</param>
        /// <param name="OrderFiled">排序的字段</param>
        /// <returns></returns>
        public List<T> Page(int PageIndex, int PageSize, out  long RowCounts, Expression<Func<T, bool>> expression = null, bool IsAsc = true, params string[] OrderFiled)
        {
            MongoCursor<T> mongoCursor;

            //条件选择
            if (expression != null)
            {
                RowCounts = collection.Find(Query<T>.Where(expression)).Count();
                mongoCursor = collection.Find(Query<T>.Where(expression));
            }
            else
            {
                RowCounts = collection.FindAll().Count();
                mongoCursor = collection.FindAll();
            }

            //排序
            if (OrderFiled != null && OrderFiled.Length > 0)
            {
                //处理主键字段
                for (int i = 0; i < OrderFiled.Length; i++)
                {
                    if (OrderFiled[i].Equals("id", StringComparison.CurrentCultureIgnoreCase))
                    {
                        OrderFiled[i] = "_id";
                    }
                }

                if (IsAsc)
                {
                    mongoCursor = mongoCursor.SetSortOrder(SortBy.Ascending(OrderFiled));
                }
                else
                {
                    mongoCursor = mongoCursor.SetSortOrder(SortBy.Descending(OrderFiled));
                }
            }

            return mongoCursor.SetSkip((PageIndex - 1) * PageSize).SetLimit(PageSize).ToList();
        }

        #region 效率低，暂时不用
        ///// <summary>
        ///// 分页
        ///// </summary>
        ///// <returns></returns>
        //public List<T> Page(int PageIndex, int PageSize, out  long RowCounts, Expression<Func<T, bool>> expression = null)
        //{
        //    List<T> ret = new List<T>();
        //    IQueryable<T> queryable;
        //    //条件选择
        //    if (expression != null)
        //    {
        //        queryable = collection.Find(Query<T>.Where(expression)).AsQueryable();
        //    }
        //    else
        //    {
        //        queryable = collection.FindAll().AsQueryable();
        //    }
        //    RowCounts = queryable.Count();
        //    ret = queryable.Skip((PageIndex - 1) * PageSize).Take(PageSize).ToList();
        //    return ret;
        //}

        ///// <summary>
        ///// 分页
        ///// </summary>
        ///// <typeparam name="TKey"></typeparam>
        ///// <param name="PageIndex"></param>
        ///// <param name="PageSize"></param>
        ///// <param name="RowCounts"></param>
        ///// <param name="expression"></param>
        ///// <param name="orderBy"></param>
        ///// <param name="IsOrder"></param>
        ///// <returns></returns>
        //public List<T> Page<TKey>(int PageIndex, int PageSize, out  long RowCounts, Expression<Func<T, bool>> expression = null, Expression<Func<T, TKey>> orderBy = null, bool IsOrder = true)
        //{
        //    List<T> ret = new List<T>();
        //    IQueryable<T> queryable;

        //    //条件选择
        //    if (expression != null)
        //    {
        //        queryable = collection.Find(Query<T>.Where(expression)).AsQueryable();
        //    }
        //    else
        //    {
        //        queryable = collection.FindAll().AsQueryable();
        //    }
        //    //排序
        //    if (orderBy != null)
        //    {
        //        if (IsOrder)
        //        {
        //            queryable = queryable.OrderBy(orderBy);
        //        }
        //        else
        //        {
        //            queryable = queryable.OrderByDescending(orderBy);
        //        }
        //    }
        //    RowCounts = queryable.Count();
        //    ret = queryable.Skip((PageIndex - 1) * PageSize).Take(PageSize).ToList();
        //    return ret;
        //} 
        #endregion

        #endregion

        #region 删除


        /// <summary>
        /// 带条件的删除
        /// </summary>
        /// <param name="expression"></param>
        /// <returns></returns>
        public void Delete(Expression<Func<T, bool>> expression)
        {
            MongoDB.Driver.IMongoQuery query = Query<T>.Where(expression);
            var result = collection.Remove(query);
        }
        /// <summary>
        /// 根据模型删除
        /// </summary>
        /// <param name="model"></param>
        public void Delete(T model)
        {
            MongoDB.Driver.IMongoQuery query = Query<T>.Where(p => p.Id == model.Id);
            collection.Remove(query);
        }

        /// <summary>
        /// 根据ID删除
        /// </summary>
        /// <param name="Id"></param>
        public void Delete(string Id)
        {
            MongoDB.Driver.IMongoQuery query = Query<T>.Where(p => p.Id == Id);
            collection.Remove(query);
        }

        /// <summary>
        /// 全部删除
        /// </summary>
        /// <returns></returns>
        public void DeleteAll()
        {
            var result = collection.RemoveAll();
        }
        #endregion

        #region 添加
        /// <summary>
        /// 单模型添加
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public void Insert(T model)
        {
            var result = collection.Insert<T>(model);
        }

        /// <summary>
        /// 批量添加
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public void InsertBatch(List<T> model)
        {
            collection.InsertBatch<T>(model);

        }
        #endregion

        #region 修改
        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public void Update(T model)
        {
            var result = collection.Save<T>(model);
        }

        /// <summary>
        /// 批量修改
        /// </summary>
        /// <param name="model"></param>
        public void UpdateAll(List<T> model)
        {
            model.ForEach(e => collection.Save<T>(e));
        }
        #endregion

    }
}
