﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Entity;
using Data.NoSql;
using System.Collections.Generic;
using System.Xml;
using System.Xml.Linq;

namespace UnitTestProject
{
    [TestClass]
    public class UnitTest1
    {
        Random rd = new Random();
        UserServices ubll = new UserServices();

        #region 添加
        [TestMethod]
        public void 添加()
        {
            UserEntity model = new UserEntity();
            model.UserName = "Name" + rd.Next(100, 10000);
            model.Num = rd.Next(100, 10000);
            model.PostTime = DateTime.Now;
            ubll.Insert(model);

        }

        [TestMethod]
        public void 添加复杂模型()
        {
            ComplexEntity model = new ComplexEntity();
            ComplexServices cbll = new ComplexServices();
            model.Name = "Complex";
            model.Schools = new List<School>();
            model.Schools.Add(new School()
            {
                Master = new Grade()
                {
                    Name = "Master"
                },
                Name = "School",
                Students = new List<Student>()
            });
            model.Schools[0].Students.Add(new Student() { Age = 22, Name = "张三" });
            cbll.Insert(model);
        }

        [TestMethod]
        public void 批量添加()
        {
            List<UserEntity> Data = new List<UserEntity>();
            for (int i = 0; i < 1000000; i++)
            {
                UserEntity model = new UserEntity();
                model.UserName = "Name" + rd.Next(100, 10000);
                model.Num = rd.Next(100, 10000);
                model.PostTime = DateTime.Now;
                Data.Add(model);
            }
            ubll.InsertBatch(Data);
        }
        #endregion

        #region 修改
        [TestMethod]
        public void 获取单个对象_修改()
        {
            var model = ubll.FirstOrDefault(p => p.Id != "");
            model.UserName = "new1";
            ubll.Update(model);
        }
        [TestMethod]
        public void 批量修改()
        {
            var model = ubll.FindAll();
            for (int i = 0; i < model.Count; i++)
            {
                model[i].UserName = "Text";
            }
            ubll.UpdateAll(model);
        }
        #endregion

        #region 查询
        [TestMethod]
        public void 获取全部对象()
        {
            var model = ubll.FindAll();
            var count = model.Count;
        }

        [TestMethod]
        public void 获取单个对象()
        {
            var model = ubll.FirstOrDefault();
            var count = model.PostTime;
        }

        [TestMethod]
        public void 根据ID获取对象()
        {
            var model = ubll.GetModelById("eeef22d6-7ac6-40cd-9312-59ab15fd904a");
        }


        [TestMethod]
        public void 获取全部对条件象_带条件()
        {
            var model = ubll.FindAll(p => p.UserName.Contains("Name"));
            var count = model.Count;
        }

        [TestMethod]
        public void 分页()
        {

            long Rows;
            List<UserEntity> pageDate = new List<UserEntity>();
            pageDate = ubll.Page(300, 20, out Rows, p => p.Num > 1500);
            pageDate = ubll.Page(1, 20, out Rows, null, true, "Id");
            pageDate = ubll.Page(1, 20, out Rows, null, true, "Num");
            pageDate = ubll.Page(1, 20, out Rows, p => p.Num > 1500, false, "Id");

        }


        [TestMethod]
        public void 获取数量()
        {
            //不带条件
            var count = ubll.GetCount();
            //带条件
            var count1 = ubll.GetCount(p => p.Num > 5000);
        }
        #endregion

        #region 删除
        [TestMethod]
        public void 删除_自定义条件()
        {
            ubll.Delete(p => p.Num >= 2000);
        }
        [TestMethod]
        public void 删除_删除模型()
        {
            var model = ubll.FirstOrDefault();
            if (model != null)
            {
                ubll.Delete(model);
            }
        }

        [TestMethod]
        public void 删除_根据ID删除()
        {
            ubll.Delete("ec45ea8b-a551-46eb-ad58-1b4f5f2aab25");
        }
        [TestMethod]
        public void 删除_删除全部()
        {
            ubll.DeleteAll();
        }
        #endregion

        #region 其他
        [TestMethod]
        public void 同时创建两个对象_同一数据库内()
        {
            LogServices Logbll = new LogServices();

            UserEntity model = new UserEntity();
            model.UserName = "Name" + rd.Next(100, 10000);
            model.Num = rd.Next(100, 10000);
            model.PostTime = DateTime.Now;
            ubll.Insert(model);

            LogEntity log = new LogEntity();
            log.UserName1 = "Name" + rd.Next(100, 10000);
            log.Num1 = rd.Next(100, 10000);
            log.PostTime1 = DateTime.Now;
            Logbll.Insert(log);


            model.UserName = "Name" + rd.Next(100, 10000);
            model.Num = rd.Next(100, 10000);
            model.PostTime = DateTime.Now;
            ubll.Insert(model);
        }

        [TestMethod]
        public void 同时创建两个对象_不同一数据库内()
        {
            Log1Services Logbll = new Log1Services();

            UserEntity model = new UserEntity();
            model.UserName = "Name" + rd.Next(100, 10000);
            model.Num = rd.Next(100, 10000);
            model.PostTime = DateTime.Now;
            ubll.Insert(model);

            LogEntity log = new LogEntity();
            log.UserName1 = "Name" + rd.Next(100, 10000);
            log.Num1 = rd.Next(100, 10000);
            log.PostTime1 = DateTime.Now;
            Logbll.Insert(log);


            model.UserName = "Name" + rd.Next(100, 10000);
            model.Num = rd.Next(100, 10000);
            model.PostTime = DateTime.Now;
            ubll.Insert(model);
        }

        [TestMethod]
        public void 当指定名称不存在时候()
        {
            ErrorServices error = new ErrorServices();
        }
        #endregion

    }
}
